/*Hearder-file for MEGSV4.DLL
Copyright ME-Messysteme GmbH H.Kabelitz*/
//return-codes
#define GSV_OK 0
#define GSV_ERROR -1
#define GSV_TRUE 1
//This appl. Errorcodes. Retrieve with GSV4getLastError
#define OWN_ERR_MASK 0x20000000 //Bit29=1, s.http://msdn.microsoft.com/en-us/library/ms680347(VS.85).aspx
#define ERR_MUTEXFAILED 0x300000F0	//d805306608
#define ERR_EVENTFAILED 0x300000F1	//d805306609
#define ERR_PAR_NO 0x300000F2		//d805306610
#define ERR_MEM_ALLOC	0x300000F3	//d805306611 eventuell zuwenig RAM
#define ERR_NO_GSV4_FOUND 0x300000F4	//d805306612 Comport konnte zwar geoeffnet werden, aber kein GSV4 antwortete
#define ERR_BYTES_WRITTEN 0x300000F5	//d805306613
#define ERR_WRONG_PARAMETER 0x30000100	//d805306624
#define ERR_NO_GSV_ANSWER 0x30000058	//d805306456
#define ERR_WRONG_ANSWER_NUM 0x30000059	//d805306457
#define ERR_WRONG_ANSWER 0x30000060	//d805306458
#define ERR_WRONG_FRAME_SUFFIX	0x30000061
#define ERR_NOT_SUPPORTED 0x30000062	//Firmware-Version zu alt
#define ERR_WRONG_COMNO 0x30000101	//d805306625
#define ERR_COM_ALREADY_OPEN	0x300000F6	//d805306614
#define ERR_COM_GEN_FAILURE 0x3000001F //hardware-or driver-error of COMport. See: ERROR_GEN_FAILURE @msdn.microsoft.com/en-us/library/ms681382(VS.85).aspx

/*Enum-Values for the type-parameter of GSV4setDigitalType and GSV4getDigitalType.
Do not or together, only single values possible for each DigIO_No,
or the same value for all 8 digital IOs with DigIO_No =0.*/
typedef enum DigIOtype_
{
	TypeInput, TypeOutput, TypeGetVal, TypeZeroAll, TypeZeroCh1, TypeZeroCh2, TypeZeroCh3, TypeZeroCh4,
	TypeThresNo1, TypeThresNo2, TypeThresNo3, TypeThresNo4, TypeThresNo5, TypeThresNo6, TypeThresNo7, TypeThresNo8,
	TypeSyncSlave, TypeSyncMaster
} DigIOtype;

#define DIGIO_TYPE_INPUT 0
#define DIGIO_TYPE_OUTPUT 1
#define DIGIO_TYPE_GETVAL 2
#define DIGIO_TYPE_ZERO_ALL 3
#define DIGIO_TYPE_ZERO_CH1 4
#define DIGIO_TYPE_ZERO_CH2 5
#define DIGIO_TYPE_ZERO_CH3 6
#define DIGIO_TYPE_ZERO_CH4 7
#define DIGIO_TYPE_THRESNO_1 8
#define DIGIO_TYPE_THRESNO_2 9
#define DIGIO_TYPE_THRESNO_3 10
#define DIGIO_TYPE_THRESNO_4 11
#define DIGIO_TYPE_THRESNO_5 12
#define DIGIO_TYPE_THRESNO_6 13
#define DIGIO_TYPE_THRESNO_7 14
#define DIGIO_TYPE_THRESNO_8 15
#define DIGIO_TYPE_SYNCSLAVE	16
#define DIGIO_TYPE_SYNCMASTER	17

/*Enum-Values for the type-parameter of GSV4set/getAnalogueType.
Do not or together. See description below. */
typedef enum AinType_
{
	AinTypeBridge2=1, AinTypeBridge10=2, AinTypeVolt5=3, AinTypePT1000=4, AinType_K=6, AinTypeVolt10=7
} AinType;

#define UNIT_NO_MAX	29
#define THRES_NO_MAX	8
#define CANID_NO_MIN 1
#define CANID_NO_MAX 8

#define ACTEX_FLAG_HANDSHAKE	4	//enables HW-handshake in in GSV4activateExtended
#define ACTEX_FLAG_WAIT_EXTENDED	0x0100	//waits longer for device-answer in GSV4activateExtended
#define ACTEX_FLAG_STOP_TX	0x0200	//stops continious data transmission in GSV4activateExtended

__declspec(dllexport) long GSV4dllVersion(void);
/*Opens a GSV4 on COM-port specified by parameters, with all necessary ressources. Returns GSV_OK,if successful or GSV_ERROR*/
__declspec(dllexport) int GSV4actExt(int ComNo, long Bitrate, long BufSize);

/*Same as above, but the following flags can be used:
ACTEX_FLAG_HANDSHAKE: if set, opens the port with hardware-handshake (rts-cts)
ACTEX_FLAG_WAIT_EXTENDED: 	//waits longer for device-answer in GSV4activateExtended
#define ACTEX_FLAG_STOP_TX	0x0200	//stops continious data transmission in GSV4activateExtended */
__declspec(dllexport) int GSV4activateExtended(int ComNo, long Bitrate, long BufSize, long flags);

/*Returns System Error Codes (http://msdn.microsoft.com/en-us/library/ms681381%28VS.85%29.aspx) retrieved by GetLastError(),
which was called by previously failed function or returns above defined errorcodes.
Can be called after another function returned GSV_ERROR. No GSV-Access*/
__declspec(dllexport) int GSV4getLastError(int ComNo); //http://msdn.microsoft.com/en-us/library/ms681381(VS.85).aspx

/*Clears Dll-buffers. Returns GSV_OK or GSV_ERROR. No GSV-access*/
__declspec(dllexport) int GSV4clearDLLbuffer(int ComNo);

/*return-value: Number of measuring-values in buffer. No GSV-access*/
__declspec(dllexport) int GSV4received(int ComNo, int Chan); //Chan=0: lowest number of all 4 buffers Chan=5: Highest buffer filling

/*Reads measuring values of all 4 channels. Returns GSV_OK, if at least one buffer is emty, GSV_TRUE, if values were read or GSV_ERROR.
Values are written where <out0>..<out3> point to, if GSV4readAll returns GSV_TRUE. No GSV-Access*/
__declspec(dllexport) int GSV4readAll(int ComNo, double* out0, double* out1, double* out2, double* out3);

/*Reads one value of channel specified by <Chan> (within 1..4) where <out> points to,
if GSV4read returns GSV_TRUE. Otherwise GSV_OK, if the buffer is emty,
or GSV_ERROR. Chan must be within 1 to 4. No GSV-access*/
__declspec(dllexport) int GSV4read(int ComNo, int Chan, double* out);

/*Reads whole Array of values in dll-value-buffer(s), if returned GSV_TRUE. No GSV-Access.
Reads up to <count> values and writes number of values, which were really read, into <valsread>. <count> must be >0.
If resulting <valsread> is smaller than <count>, the whole buffer (the smallest, if Chan=0) was read.
If Chan=0: Reads whole Array of all channels into the array of double-type, where <out> points to. <count> must be dividible by 4.
<out> is sorted as follows: Chan1 in n*4, Chan2 in (n*4)+1, Chan3 in (n*4)+2, Chan4 in (n*4)+3 with n= {0..(<valsread> /4)-1}
E.g. if <valsread>=8, the content of <out> is, beginning with index=0: oldCh1,oldCh2,oldCh3,oldCh4,newCh1,newCh2,newCh3,newCh4
Returns GSV_OK, if at least one buffer is emty, GSV_TRUE, if values were read or GSV_ERROR.
If Chan={1..4}: Reads whole Array of specified channel. Oldest value is at index 0 (base) of <out>-array, newest at <valsread>-1.
Returns GSV_OK, if the buffer is emty, GSV_TRUE, if value(s) were read or GSV_ERROR.*/
__declspec(dllexport) int GSV4readMultiple(int ComNo, int Chan, double *out, int count, int* valsread);

/*starts GSV-4 value transmission. State not saved in device.*/
__declspec(dllexport) int GSV4startTX(int ComNo);

/*stops GSV-4 value transmission. State not saved in device.*/
__declspec(dllexport) int GSV4stopTX(int ComNo);

/*Must be used for Cleaning dll-ressources at program-end. No GSV-Access*/
__declspec(dllexport) int GSV4release(int ComNo);

/*Tares specified Channel to 0. If Chan is =0: tares all 4 Channels, otherwise: Chan: 1..4 */
__declspec(dllexport) int GSV4setZero(int ComNo, int Chan);

/*Store GSV-4-parameters at DataNo 2 or 3. */
__declspec(dllexport) int GSV4saveAll(int ComNo, int DataNo);

/*Loads GSV-4-parameters from DataNo 2 or 3 or Default-Settings from DataNo =1.  */
__declspec(dllexport) int GSV4getAll(int ComNo, int DataNo); //DataNo: 1..3

/*Change datarate with this. If desired datarate in Freq not available, it sets next possible value. */
__declspec(dllexport) int GSV4writeFreq(int ComNo, double Freq);

/*Read datarate with this.*/
__declspec(dllexport) double GSV4readFreq(int ComNo);

/*Read Serialnumber with this. After called, serialnumber is in location where <SerNo> points to.*/
__declspec(dllexport) int GSV4readSerialNo(int ComNo, char* SerNo);

__declspec(dllexport) int GSV4writeThreshold(int ComNo, int ThresNo, double ThresOn, double ThresOff);

__declspec(dllexport) int GSV4readThreshold(int ComNo, int ThresNo, double *ThresOn, double* ThresOff);

__declspec(dllexport) int GSV4setMode(int ComNo, int Mode, char* pw);

__declspec(dllexport) int GSV4getMode(int ComNo);

/*Read Firmwareversion with this.*/
__declspec(dllexport) int GSV4firmwareVersion(int ComNo);

/* Use this to trigger single transmission of one measuring value (all 4 channels).
Mainly used, if GSV-4 value transmission is switched off by GSV4stopTX.
After called, retrieve that value with GSV4read or GSV4readAll.*/
__declspec(dllexport) int GSV4getValue(int ComNo);

/* Changes Baudrate of the GSV-4's serial interface.
Available Baudrates are:
   9600,
   19200,
   38400,
   57600,
   115200,
   230400 */
__declspec(dllexport) int GSV4setBaudSerial(int ComNo, long Bitrate);

/* Reads Baudrate of the GSV-4's serial interface.*/
__declspec(dllexport) long GSV4getBaudSerial(int ComNo);

/*Change properties of input specified by <Chan>.
<Gain> can be one of the following:
0: Channel is off.
1: Input is set to bridge-sensor, Sensitivity is 2 mV/V.
2: Input is set to bridge-sensor, Sensitivity is 10 mV/V.
3: Input is set to voltage-input, Sensitivity is +5V (0 to 5V).
4: Input is set to temperature-sensor of type PT1000.
5: Input is set to bridge-sensor, Sensitivity is 0.1 mV/V.*/
__declspec(dllexport) int GSV4setGain(int ComNo, int Chan, int gain);

/*Read properties of input specified by <Chan>. See coding above.*/
__declspec(dllexport) long GSV4getGain(int ComNo, int Chan);

/*Change properties of input specified by <Chan>.
<type> can be one of above defined enum-type "AinType":
AinTypeBridge2 : Input is set to bridge-sensor, Sensitivity is 2 mV/V.
AinTypeBridge10 : Input is set to bridge-sensor, Sensitivity is 10 mV/V.
AinTypeVolt5 : Input is set to voltage-input, Sensitivity is +5V (0 to 5V).
AinTypePT1000 : Input is set to temperature-sensor of type PT1000.
AinType_K=6 : Input is set to temperature-sensor of type K-cable
AinTypeVolt10 : Input is set to voltage-input, Sensitivity is +10V (0 to 10V).
*/
__declspec(dllexport) int GSV4setAnalogueType(int ComNo, int Chan, AinType type); //Chan=0: All channels, else  1..4

/*Reads properties of all 4 channel's inputs. If the return-value is GSV_OK, the input types are written to,
where typCh1..typCh4 point to.
typCh1..typCh4 can be one of above defined enumerated type AinType. Example of usage:

*/
__declspec(dllexport) int GSV4getAnalogueType(int ComNo, AinType* typCh1, AinType* typCh2, AinType* typCh3, AinType* typCh4);


/*Write Display-scaling to GSV-4. This value is not used by the device, but will remain stored in it.
Normally used by application-software for scaling the measuring-values by multiplying them with the value
stored previously by GSV4DispSetNorm and read by GSV4DispGetNorm.*/
__declspec(dllexport) int GSV4DispSetNorm(int ComNo, int Chan, double Norm);

/*Read Display-scaling stored in GSV-4. This value is not used by the device, but will remain stored in it.
Normally used by application-software for scaling the measuring-values by multiplying them with the value
stored previously by GSV4DispSetNorm and read by GSV4DispGetNorm.*/
__declspec(dllexport) double GSV4DispGetNorm(int ComNo, int Chan);

/* Changes Baudrate of the GSV-4's CAN interface.
Available Baudrates are in kBis/s:
   20,
   50,
   80,
   100,
   125,
   250,
   500,
   1000 */
__declspec(dllexport) int GSV4setBaudCAN(int ComNo, long Bitrate);

/* Reads Baudrate of the GSV-4's CAN interface.*/
__declspec(dllexport) long GSV4getBaudCAN(int ComNo);

/* Changes ID of the GSV-4's CAN interface. Each device has 4 different IDs, which are specified by <IDno>
and can be one of the following:
TX_1:	1
TX_2:	2
RX_1:	5
RX_2:	6 */
__declspec(dllexport) int GSV4setCanID(int ComNo, int IDno, long CanID);

/* Reads ID of the GSV-4's CAN interface. Each device has 4 different IDs, which are specified by <IDno>*/
__declspec(dllexport) long GSV4getCanID(int ComNo, int IDno);

/* Sets / reads unit-Number */
__declspec(dllexport) int GSV4setUnit(int ComNo, int Chan, long UnitNo);

__declspec(dllexport) long GSV4getUnit(int ComNo, int Chan);

__declspec(dllexport) int GSV4setUnitText(int ComNo, int Chan, char* UnitTxt);

__declspec(dllexport) int GSV4getUnitText(int ComNo, int Chan, char* UnitTxt);

__declspec(dllexport) int GSV4setDigitalIOtype(int ComNo, int DigIO_No, DigIOtype type);	//DigIO_No=0: all Bits, else: 1..8

__declspec(dllexport) int GSV4getDigitalIOtype(int ComNo, int DigIO_No);	//DigIO_No=0: all Bits, else: 1..8

__declspec(dllexport) int GSV4setDigitalIOvalue(int ComNo, int DigIO_No, int value);	//DigIO_No=0: all Bits, else: 1..8

__declspec(dllexport) long GSV4getDigitalIOvalue(int ComNo, int DigIO_No);	//DigIO_No=0: all Bits, else: 1..8

__declspec(dllexport) int GSV4setTXmode(int ComNo, int val);

__declspec(dllexport) int GSV4getTXmode(int ComNo);

__declspec(dllexport) int GSV4writeOffset(int ComNo, int Chan, int gain, long offset);

__declspec(dllexport) long GSV4readOffset(int ComNo, int Chan, int gain);
//to do: (warte auf thomas wg. FW-Korrektur & Berechnungsformel
__declspec(dllexport) int GSV4writeScaledOffset(int ComNo, int Chan, int gain, double offset);

__declspec(dllexport) double GSV4readScaledOffset(int ComNo, int Chan, int gain);

__declspec(dllexport) int MeGSV4setScale(int ComNo, int Chan, int gain, long Scale);

__declspec(dllexport) long MeGSV4getScale(int ComNo, int Chan, int gain);

__declspec(dllexport) int MeGSV4writeSerialNo(int ComNo, char* SerNo);